within ;
package glucose
  model glucoseInsulin_regulationModel
    Modelica.Blocks.Sources.Constant theta(k=2.5) "Reabsorption Maximum"
      annotation (Placement(transformation(
          extent={{-5,-5},{5,5}},
          rotation=180,
          origin={85,85})));
    Modelica.Blocks.Sources.Constant QL(k=8400) "Glucose Input Flow Rate"
      annotation (Placement(transformation(extent={{-94,20},{-84,30}})));
    Modelica.Blocks.Sources.Constant phi(k=0.51)
      "Threshold Glucose Concentration" annotation (Placement(transformation(
          extent={{-5,-5},{5,5}},
          rotation=180,
          origin={85,-15})));
    Modelica.Blocks.Math.Gain Mi(k=7200) annotation (Placement(transformation(
          extent={{-6,-6},{6,6}},
          rotation=180,
          origin={10,80})));
    Modelica.Blocks.Math.Gain Lambda(k=2470) annotation (Placement(
          transformation(
          extent={{-6,-6},{6,6}},
          rotation=180,
          origin={10,54})));
    Modelica.Blocks.Math.Gain v(k=139000) "Nu" annotation (Placement(
          transformation(
          extent={{-6,-6},{6,6}},
          rotation=0,
          origin={-50,-10})));
    Modelica.Blocks.Math.Gain CG(k=1/15000) "Capacitance of Glucose"
      annotation (Placement(transformation(
          extent={{-6,-6},{6,6}},
          rotation=0,
          origin={10,30})));
    Modelica.Blocks.Math.Gain beta(k=1430) annotation (Placement(transformation(
          extent={{-6,-6},{6,6}},
          rotation=180,
          origin={30,-10})));
    Modelica.Blocks.Math.Gain CI(k=1/15000) "Capacitance of Insulin"
      annotation (Placement(transformation(
          extent={{-6,-6},{6,6}},
          rotation=0,
          origin={30,-40})));
    Modelica.Blocks.Math.Gain alpha(k=7600) "Insulin Destruction Gain"
      annotation (Placement(transformation(
          extent={{-6,-6},{6,6}},
          rotation=180,
          origin={30,-70})));
    Modelica.Blocks.Math.Add add(k2=-1)
      annotation (Placement(transformation(extent={{4,-46},{16,-34}})));
    Modelica.Blocks.Math.Add add1(k1=-1, k2=+1) annotation (Placement(
          transformation(
          extent={{-6,-6},{6,6}},
          rotation=180,
          origin={50,-10})));
    Modelica.Blocks.Math.Product product
      annotation (Placement(transformation(extent={{-76,-16},{-64,-4}})));
    Modelica.Blocks.Math.Add add2
      annotation (Placement(transformation(extent={{-56,24},{-44,36}})));
    Modelica.Blocks.Math.MultiSum multiSum(nu=4, k={-1,-1,1,-1})
      annotation (Placement(transformation(extent={{-26,24},{-14,36}})));
    Modelica.Blocks.Math.Add add3(k1=+1, k2=-1) annotation (Placement(
          transformation(
          extent={{-6,-6},{6,6}},
          rotation=180,
          origin={30,80})));
    Modelica.Blocks.Nonlinear.Limiter ThresholdingOperator(uMax=100000, uMin=0)
      annotation (Placement(transformation(
          extent={{-6,-6},{6,6}},
          rotation=180,
          origin={-10,80})));
    Modelica.Blocks.Nonlinear.Limiter ThresholdingOperator1(uMax=100000, uMin=0)
      annotation (Placement(transformation(
          extent={{-6,-6},{6,6}},
          rotation=180,
          origin={10,-10})));
    Modelica.Blocks.Continuous.Integrator integrator(y_start=0.81)
      annotation (Placement(transformation(extent={{24,24},{36,36}})));
    Modelica.Blocks.Continuous.Integrator integrator1(y_start=0.055)
      annotation (Placement(transformation(extent={{44,-46},{56,-34}})));
    Modelica.Blocks.Interfaces.RealOutput OUT1 "Blood Glucose
Concentration
(mg/mL)" annotation (Placement(transformation(extent={{90,20},{110,40}}),
          iconTransformation(extent={{80,30},{100,50}})));
    Modelica.Blocks.Interfaces.RealOutput OUT2 "Blood Insulin
Concentration
(mU/ML)" annotation (Placement(transformation(extent={{90,-50},{110,-30}}),
          iconTransformation(extent={{80,-50},{100,-30}})));
    Modelica.Blocks.Interfaces.RealInput IN1 "Glucose Infusion Rate
(mg/hr)" annotation (Placement(transformation(extent={{-110,40},{-90,60}}),
          iconTransformation(extent={{-100,-10},{-80,10}})));
  equation
    connect(add.y, CI.u) annotation (Line(
        points={{16.6,-40},{22.8,-40}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(alpha.y, add.u2) annotation (Line(
        points={{23.4,-70},{-22,-70},{-22,-43.6},{2.8,-43.6}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(beta.u, add1.y) annotation (Line(
        points={{37.2,-10},{43.4,-10}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(add1.u1, phi.y) annotation (Line(
        points={{57.2,-13.6},{70.6,-13.6},{70.6,-15},{79.5,-15}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(product.y, v.u) annotation (Line(
        points={{-63.4,-10},{-57.2,-10}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(QL.y, add2.u2) annotation (Line(
        points={{-83.5,25},{-68.65,25},{-68.65,26.4},{-57.2,26.4}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(multiSum.y, CG.u) annotation (Line(
        points={{-12.98,30},{2.8,30}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(add3.u2, theta.y) annotation (Line(
        points={{37.2,83.6},{58.6,83.6},{58.6,85},{79.5,85}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(ThresholdingOperator.u, Mi.y) annotation (Line(
        points={{-2.8,80},{3.4,80}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(ThresholdingOperator1.u, beta.y) annotation (Line(
        points={{17.2,-10},{23.4,-10}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(add.u1, ThresholdingOperator1.y) annotation (Line(
        points={{2.8,-36.4},{-22,-36.4},{-22,-10},{3.4,-10}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(Lambda.y, multiSum.u[1]) annotation (Line(
        points={{3.4,54},{-28,54},{-28,33.15},{-26,33.15}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(ThresholdingOperator.y, multiSum.u[2]) annotation (Line(
        points={{-16.6,80},{-30,80},{-30,31.05},{-26,31.05}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(add2.y, multiSum.u[3]) annotation (Line(
        points={{-43.4,30},{-34.7,30},{-34.7,28.95},{-26,28.95}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(v.y, multiSum.u[4]) annotation (Line(
        points={{-43.4,-10},{-34,-10},{-34,26.85},{-26,26.85}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(CI.y, integrator1.u) annotation (Line(
        points={{36.6,-40},{42.8,-40}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(integrator1.y, alpha.u) annotation (Line(
        points={{56.6,-40},{68,-40},{68,-70},{37.2,-70}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(integrator1.y, product.u2) annotation (Line(
        points={{56.6,-40},{68,-40},{68,-84},{-82,-84},{-82,-13.6},{-77.2,-13.6}},
        color={0,0,127},
        smooth=Smooth.None));

    connect(CG.y, integrator.u) annotation (Line(
        points={{16.6,30},{22.8,30}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(integrator.y, Lambda.u) annotation (Line(
        points={{36.6,30},{60,30},{60,54},{17.2,54}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(integrator.y, product.u1) annotation (Line(
        points={{36.6,30},{60,30},{60,10},{-82,10},{-82,-6.4},{-77.2,-6.4}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(integrator.y, add1.u2) annotation (Line(
        points={{36.6,30},{60,30},{60,-6.4},{57.2,-6.4}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(integrator.y, add3.u1) annotation (Line(
        points={{36.6,30},{60,30},{60,76.4},{37.2,76.4}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(integrator.y, OUT1) annotation (Line(
        points={{36.6,30},{100,30}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(integrator1.y, OUT2) annotation (Line(
        points={{56.6,-40},{100,-40}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(add2.u1, IN1) annotation (Line(
        points={{-57.2,33.6},{-68,33.6},{-68,50},{-100,50}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(Mi.u, add3.y) annotation (Line(
        points={{17.2,80},{23.4,80}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(OUT1, OUT1) annotation (Line(
        points={{100,30},{100,30}},
        color={0,0,127},
        smooth=Smooth.None));
    annotation (Diagram(graphics={Text(
            extent={{-38,82},{-30,80}},
            lineColor={0,0,0},
            fillColor={215,215,215},
            fillPattern=FillPattern.Solid,
            textString="RLR")}), Icon(graphics={
          Rectangle(
            extent={{-100,100},{100,-100}},
            lineColor={0,0,0},
            fillColor={215,215,215},
            fillPattern=FillPattern.Solid),
          Text(
            extent={{-80,80},{80,60}},
            lineColor={0,0,0},
            textString="Glucose-Insulin"),
          Text(
            extent={{-80,40},{80,20}},
            lineColor={0,0,0},
            textString="Regulation model")}));
  end glucoseInsulin_regulationModel;

  model glucoseBlock
    Modelica.Blocks.Sources.Pulse GlucoseInfusion(
      amplitude=100000,
      width=5,
      period=5,
      startTime=0.5)
      annotation (Placement(transformation(extent={{-60,-10},{-40,10}})));
    glucoseInsulin_regulationModel glucoseInsulin_regulationModel1
      annotation (Placement(transformation(extent={{0,-10},{20,10}})));
  equation
    connect(GlucoseInfusion.y, glucoseInsulin_regulationModel1.IN1) annotation (
       Line(
        points={{-39,0},{1,0}},
        color={0,0,127},
        smooth=Smooth.None));
    annotation (Diagram(graphics));
  end glucoseBlock;


  annotation (uses(Modelica(version="3.2")));
end glucose;
